/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

// Some constants
if(typeof(CC) == "undefined") const CC = Components.classes;  
if(typeof(CI) == "undefined") const CI = Components.interfaces; 
if(typeof(CR) == "undefined") const CR = Components.results;
if(typeof(CU) == "undefined") const CU = Components.utils;

if(typeof(XUL_NS) == "undefined") 
	const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

if(typeof(XHTML_NS) == "undefined") 
	const XHTML_NS = "http://www.w3.org/1999/xhtml";

// Some URLs to icons
const TBNOTES_MSGHASNOTE_ICON_URL = "chrome://tbnotes/skin/icons/note-flag16x16.png";
const TBNOTES_MSGHASNONOTE_ICON_URL = "chrome://tbnotes/skin/icons/note-no-flag16x16.png";

// Note dialog constants
const TBNOTES_NOTEDLG_URI = "chrome://tbnotes/content/note-dialog.xul";
const TBNOTES_NOTEDLG_FTRS = "chrome,extrachrome,dialog,toolbar,modal,resizable=yes";

// Link dialog constants
const TBNOTES_LINKDLG_URI = "chrome://tbnotes/content/link-dialog.xul";
const TBNOTES_LINKDLG_FTRS = "chrome,extrachrome,dialog,toolbar,modal,resizable=yes";

// Main tbNotes preferences key
const TBNOTES_PREFS_KEY = "extensions.tbnotes.";

// URL of all notes view
const TBNOTES_NOTESVIEW_URI = "chrome://tbnotes/content/notes-view/notes-view.xhtml";

// Needed for access to tbNotes preferences
try {
	const TBNOTES_PREFS = CC["@mozilla.org/preferences-service;1"].
			getService(CI.nsIPrefBranch);
} catch(e) {
	CU.reportError("tbNotes: error when initializing preferences service!");
}

// Namespace 
if(typeof(extensions) == "undefined") var extensions = {};
if(typeof(extensions.tbnotes) == "undefined") extensions.tbnotes = {};

// And some basic utility functions
(function() {
  
	/**
	 * Helper function that returns current date and time (e.g. "2009-09-16 14:33:00").
	 * 
	 * @return string
	 */
	this.getCurrentDatetime = function() 
	{
		let d = new Date();
		let now = d.getFullYear() + "-" + 
						((d.getMonth()<10) ? "0" + d.getMonth() : d.getMonth()) + "-" + 
						((d.getDate()<10) ? "0" + d.getDate() : d.getDate()) + " " + 
						((d.getHours()<10) ? "0" + d.getHours() : d.getHours()) + ":" + 
						((d.getMinutes()<10) ? "0" + d.getMinutes() : d.getMinutes()) + ":00";
		
		return now;
	}; // end getCurrentDatetime()
	
	/**
	 * Removes all child nodes of given element.
	 * 
	 * @param aElm DOMElement
	 * @return void
	 */
	this.removeChilds = function(aElm) {
		try {
			var ch = aElm.lastChild;
			while(ch) {
				aElm.removeChild(ch);
				ch = aElm.lastChild;
			}
		} catch(e) {
			CU.reportError(e);
		}
	}; // end removeChilds(aElm)

	/**
	 * Checks if given URL is valid
	 * 
	 * @see http://www.codetoad.com/forum/15_24179.asp
	 * @param aUrl {string}
	 * @returns {boolean}
	 */
	this.checkUrl = function(aUrl)
	{
		var lengthVal = trim(aUrl);
		lengthVal = lengthVal.length;

		if(lengthVal != 0) {
			var j = new RegExp();
			j.compile("^[A-Za-z]+://[A-Za-z0-9-]+\.[A-Za-z0-9]+");
			lengthVal = trim(aUrl);

			if(!j.test(lengthVal)) {
				return false;
			}
		}

		return true;
	}; // end checkUrl(aUrl)

	/**
	 * Copy given text to the system clipboard
	 *
	 * @param aText {string}
	 * @returns {boolean}
	 */
	this.copyTextToClipboard = function(aText)
	{
		try {
			let clip = CC["@mozilla.org/widget/clipboard;1"].createInstance(CI.nsIClipboard);
			let trans = CC["@mozilla.org/widget/transferable;1"].createInstance(CI.nsITransferable);
			let str = CC["@mozilla.org/supports-string;1"].createInstance(CI.nsISupportsString);
			
			str.data = aText;
			trans.addDataFlavor("text/unicode");
			trans.setTransferData("text/unicode", str, aText.length * 2);
			clip.setData(trans, null, CI.nsIClipboard.kGlobalClipboard);

			return true;
		} catch(e) {
			CU.reportError(e);
			
			return false;
		}
	}; // end copyTextToClipboard(aText)
	
	/**
	 * Opens link in the system browser.
	 *
	 * @param aUrl {string}
	 */
	this.openLinkInExternalBrowser = function(aUrl)
	{
		try {
			var ioSrv = CC["@mozilla.org/network/io-service;1"].getService(CI.nsIIOService);
			var uri = ioSrv.newURI(aUrl, null, null);
			var extps = CC["@mozilla.org/uriloader/external-protocol-service;1"].
					getService(CI.nsIExternalProtocolService);

			extps.loadURI(uri, null);
		} catch(e) {
			CU.reportError(e);
		}
	}; // end openLinkInExternalBrowser(aUrl)
  
}).apply(extensions.tbnotes);
