/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Main controller object of link dialog
 * 
 * @type Object
 */
var tbLinkDialog = {
	
	/**
	 * Holds object of edited link.
	 * 
	 * @type extensions.tbnotes.data.LinkPrototype|NULL
	 */
	_editedLink: null,
	
	/**
	 * Fired when dialog is loading.
	 * 
	 * @return void
	 */
	onLoad: function() 
  {
		let op = "add";
		
		if(typeof(window.arguments[0].inn) != "undefined") {
		    if(window.arguments[0].inn != null) {
		        this._editedLink = window.arguments[0].inn;
		    }
		}
		
		this._updateUi();
	},
	
	/** 
	 * Updates dialog UI.
	 * 
	 * @return void
	 */
	_updateUi: function() 
    {
		var dlg = document.getElementById("tbnotes-link-dialog");
		var title = document.getElementById("tblinkdialog_title-textbox");
		var url = document.getElementById("tblinkdialog_url-textbox");
		
		if(this._editedLink != null) {
			// We are editing a link
			title.value = this._editedLink.mTitle;
			url.value = this._editedLink.mUrl;
			
			var title = dlg.getAttribute("titleEdit");
			if(typeof(this._editedLink.mId) != "undefined") {
				title = title + " #" + this._editedLink.mId;
			}
			
			dlg.setAttribute("title", title);
		} else {
			// We are adding a new link
			dlg.setAttribute("title", dlg.getAttribute("titleAdd"));
		}
		
		// Select title textbox
		title.focus();
	},
	
	/** 
	 * Fired when user accepted this dialog.
	 * 
	 * @return boolean
	 */
	onAccept: function() {
		try {
			var title = document.getElementById("tblinkdialog_title-textbox");
			var url = document.getElementById("tblinkdialog_url-textbox");
			
			// TODO Check if title is empty!
			// TODO Check if URL is empty!
			
			if(this._editedLink == null) {
				let noteObj = null;
				let winMan = CC["@mozilla.org/appshell/window-mediator;1"].getService();
				let winManInt = winMan.QueryInterface(CI.nsIWindowMediator);
				let win = winManInt.getMostRecentWindow("mail:3pane");
				
				with(win) {
					noteObj = new extensions.tbnotes.data.LinkPrototype();
				}
				
				// We are adding a new link
				this._editedLink = noteObj;
				this._editedLink.mTitle = title.value;
				this._editedLink.mUrl = url.value;
			}
			else {
				// We are editing note
				this._editedLink.mTitle = title.value;
				this._editedLink.mUrl = url.value;
			}
	
			window.arguments[0].out = this._editedLink;
		} catch(e) { 
		    CU.reportError(e); 
		}
		
		return true;
	}, 
	
	/**
	 * Fired when user canceled this dialog.
	 * 
	 * @return boolean
	 */
	onCancel: function() {
		var pref = TBNOTES_PREFS_KEY + "dont_show_close_confirm";
		
		// Check if user want to see confirmation dialog
		if(TBNOTES_PREFS.prefHasUserValue(pref) === true) {
			if(TBNOTES_PREFS.getBoolPref(pref) === true) {
				return true;
			}
		}
		
		// Show confirmation if user really want to close unsaved note
		var prompts = CC["@mozilla.org/embedcomp/prompt-service;1"].
				getService(CI.nsIPromptService);
		
		// TODO Use stringbundles!
		var title = "Link dialog";
		var desc = "Do you really want to close unsaved link?";
		var question = "Do not ask me again.";
		var check = { value: false };
		var res = prompts.confirmCheck(window, title, desc, question, check);
		
		TBNOTES_PREFS.setBoolPref(pref, check.value);
		
		return res;
	}
	
}; // End of tbLinkDialog
