/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Main controller object of note dialog
 * 
 * @type Object
 */
var tbNoteDialog = {
	
	/** 
	 * Holds instance of object describing target email message
	 * @type Object|NULL
	 */
	_targetMsg: null,
	
	/**
	 * Holds instance of object representing edited note.
	 * @type Object|NULL
	 */
	_editedNote: null,
	
	/**
	 * Fired when dialog is loading.
	 * 
	 * @return void
	 */
	onLoad: function() 
  {
		let op = "add";
		
		if(typeof(window.arguments[0].inn.note) != "undefined") {
			if(window.arguments[0].inn.note != null) {
				op = "edit";
			}
		}
		
		if(op == "add") {
			// Check if we targeting note to some message
			// TODO When we editing note target message can be blank so load subject by self here!
			if(typeof(window.arguments[0].inn.msg) != "undefined") {
				if(window.arguments[0].inn.msg != null) {
					this._targetMsg = window.arguments[0].inn.msg;
				}
			}
			
			this._noteLinks = new Array();
		}
		else {
			this._editedNote = window.arguments[0].inn.note;
			this._noteLinks = this._editedNote.getLinks();
		}
		
		this.updateDialog();
	},
	
	/** 
	 * Updates dialog's UI.
	 * 
	 * @return void
	 */
	updateDialog: function() 
  {
		var dlg = document.getElementById("tbnotes-note-dialog");
		var cont = document.getElementById("tbnotedialog_createdCont-hbox");
		var target = document.getElementById("tbnotedialog_target-textbox");
		var created = document.getElementById("tbnotedialog_created-textbox");
		var title = document.getElementById("tbnotedialog_title-textbox");
		var body = document.getElementById("tbnotedialog_body-textbox");
		
		if(this._editedNote != null) {
			// We are editing given note
			target.value = this._editedNote.mTargetName;
			target.setAttribute("tooltiptext", this._editedNote.mTargetName);
			created.value = this._editedNote.mCreated;
			title.value = this._editedNote.mTitle;
			body.value = this._editedNote.mBody;
			
			if(cont.hasAttribute("collapsed")) {
				cont.removeAttribute("collapsed");
			}
			
			// TODO Use stringbundles!
			var dlgTitle = dlg.getAttribute("titleEdit");
			if(typeof(this._editedNote.mId) != "undefined") {
				dlgTitle = dlgTitle + " #" + this._editedNote.mId;
			}
			dlg.setAttribute("title", dlgTitle);
		} else {
			// We are adding new note
			
			// Has note any target?
			if(this._targetMsg != null) {
				target.value = this._targetMsg.subject;
				target.setAttribute("tooltiptext", this._targetMsg.subject);
			} else {
				// TODO Use stringbundles!
				target.value = "No target";
			}
			
			// Hide grid row with created textbox
			cont.setAttribute("collapsed", "true");
			dlg.setAttribute("title", dlg.getAttribute("titleAdd"));
		}
		
		this.updateLinks();
		
		// Select title textbox
		title.focus();
	},
	
	/**
	 * Update links UI
	 *
	 * @returns {void}
	 */
	updateLinks: function()
	{
		var links = document.getElementById("tbnotedialog_links-description");
		// Remove current links
		extensions.tbnotes.removeChilds(links);
		
		if(this._noteLinks.length == 0) {
			// Append 'No links attached' label
			let label = document.createElementNS(XUL_NS, "label");
			// TODO Use stringbundles!
			label.setAttribute("value", "No links attached yet");
			label.setAttribute("class", "noLinksAttached");
			links.appendChild(label);
		}
		else {
			// TODO Display all links
			for(let i=0; i<this._noteLinks.length; i++) {
				let label = document.createElementNS(XUL_NS, "label");
				label.setAttribute("value", this._noteLinks[i].mTitle);
				// TODO Use stringbundles!
				label.setAttribute("tooltiptext", "URL: " + this._noteLinks[i].mUrl);
				label.setAttribute("class", "link editLinkCmd");
				//label.setAttribute("popup", "tbnotedialog_link-popup");
				label.setAttribute("onclick", "tbNoteDialog.onLinkClick(event, '" + i + "');");
				links.appendChild(label);
			}
		}
	},
	
	/** 
	 * Fired when user accepted the note dialog.
	 * 
	 * @return boolean
	 */
	onAccept: function()
	{
		try {
			var titleElm = document.getElementById("tbnotedialog_title-textbox");
			var bodyElm = document.getElementById("tbnotedialog_body-textbox");
			
			if(titleElm.value == "") {
				// TODO Use stringbundles!
				alert("It is not allowed to create note without a title. Please, enter any title or cancel dialog.")
			}
			
			// TODO What if target was cleared by user?
			
			if(this._editedNote == null) {
				var noteObj = null;
				var win = this.getMessengerWin();
				with(win) {
					noteObj = new extensions.tbnotes.data.NotePrototype();
				}
				
				// We are adding new note
				this._editedNote = noteObj;
				
				if(this._targetMsg != null) {
					var isNews = this._targetMsg.selectedMessageIsNews;
					var isFeed = this._targetMsg.selectedMessageIsFeed;
					
					if(isNews) {
						this._editedNote.mTargetType = "newsitem";
					} else if(isFeed) {
						this._editedNote.mTargetType = "feeditem";
					} else {
						this._editedNote.mTargetType = "mailmessage";
					}
					// TODO `address book items` => `abitem`
					
					this._editedNote.mTargetKey = this._targetMsg.key;
					this._editedNote.mTargetName = this._targetMsg.subject;
				}
			}
			
			this._editedNote.mTitle = titleElm.value;
			this._editedNote.mBody = bodyElm.value;
			this._editedNote.setLinks(this._noteLinks);
			
			window.arguments[0].out = this._editedNote;
		} catch(e) {
			CU.reportError(e);
		}
		
		return true;
	}, 
	
	/**
	 * Fired when user canceled the note dialog.
	 * 
	 * @return boolean
	 */
	onCancel: function()
	{
		var ret = true;
		
		let win = this.getMessengerWin();
		with(win) {
			var pref = TBNOTES_PREFS_KEY + "dont_show_close_confirm";
			
			// Check if user want to see confirmation dialog
			if(TBNOTES_PREFS.prefHasUserValue(pref) === true) {
				if(TBNOTES_PREFS.getBoolPref(pref) === true) {
					return true;
				}
			}
			
			// Show confirmation if user really want to close unsaved note
			var prompts = CC["@mozilla.org/embedcomp/prompt-service;1"].
					getService(CI.nsIPromptService);
			
			// TODO Use stringbundles!
			var title = "Note dialog";
			var desc = "Do you really want to close unsaved note?";
			var question = "Do not ask me again.";
			var check = { value: false };
			ret = prompts.confirmCheck(window, title, desc, question, check);
			
			TBNOTES_PREFS.setBoolPref(pref, check.value);
		}
		
		return res;
	},
	
	/**
	 * Performs requested command
	 * 
	 * @param aCmdName {string}
	 * @return void
	 */
	doCommand: function(aCmdName)
	{
		var win = this.getMessengerWin();
		
		switch(aCmdName) {
			// Add new link command
			case "add_link":
				with(win) {
					var params = { inn: null, out: null };
					window.openDialog(TBNOTES_LINKDLG_URI, "", TBNOTES_LINKDLG_FTRS, params).focus();
					
					if(params.out) {
						var link = new extensions.tbnotes.data.LinkPrototype();
						link.mTitle = params.out.mTitle;
						link.mUrl = params.out.mUrl;
						
						this._noteLinks.push(link);
						this.updateLinks();
					}
				}
				break;
			
			// Edit link
			case "edit_link":
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link == null) return;
						
					var params = { inn: link, out: null };
					window.openDialog(TBNOTES_LINKDLG_URI, "", TBNOTES_LINKDLG_FTRS, params).focus();
					
					if(params.out) {
						this._noteLinks[this._link_index].mTitle = params.out.mTitle;
						this._noteLinks[this._link_index].mUrl = params.out.mUrl;
						this.updateLinks();
					}
				}
				break;
			
			// Remove selected link
			case 'remove_link':
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link == null) return;
					
					var pref = TBNOTES_PREFS_KEY + "dont_show_remove_confirm";
					
					// Check if user want to see confirmation dialog
					if(TBNOTES_PREFS.prefHasUserValue(pref) === true) {
						if(TBNOTES_PREFS.getBoolPref(pref) === true) {
							// Show confirmation if user really want to close unsaved note
							var prompts = CC["@mozilla.org/embedcomp/prompt-service;1"].
									getService(CI.nsIPromptService);
							
							// TODO Use stringbundles!
							var title = "Note dialog";
							var desc = "Do you really want to remove link '" + link.mTitle + "'?";
							var question = "Do not ask me again.";
							var check = { value: false };
							var res = prompts.confirmCheck(window, title, desc, question, check);
							
							TBNOTES_PREFS.setBoolPref(pref, check.value);
							
							if(res) {
								link.remove();
								this._noteLinks.splice(this._link_index, 1);
								this.updateLinks();
							}
						}
					} else {
						link.remove();
						this._noteLinks.splice(this._link_index, 1);
						this.updateLinks();
					}
				}
				break;
			
			// Show link in the browser
			case 'show_link':
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link != null) {
						extensions.tbnotes.openLinkInExternalBrowser(link.mUrl);
					}
				}
				break;
			
			// Copy URL of the selected link to the clipboard
			case 'copy_link':
				with(win) {
					var link = this._noteLinks[this._link_index];
					
					if(link != null) {
						extensions.tbnotes.copyTextToClipboard(link.mUrl);
					}
				}
				break;
			
			default:
				CU.reportError("tbNoteDialog.doCommand:: undefined command '" + aCmdName + "'.");
				break;
		}
	},
	
	/**
	 * Fired when user clicked on any link. It shows the link's context menu.
	 *
	 * @param aEvent {Components.interfaces.nsIDOMEvent}
	 * @param aLinkIndex {integer} Index of link in tbNotesDialog._noteLinks
	 * @returns void
	 */
	onLinkClick: function(aEvent, aLinkIndex) {
		// We can not use link.mId here!
		this._link_index = aLinkIndex;
		
		var popup = document.getElementById("tbnotedialog_link-popup");
		popup.openPopup(aEvent.target, "after_pointer", null, null, true);
	},
	
	/**
	 * Fired when link's context menu is going to be hidden. It erases ID of the link
	 * from tbNotesDialog object.
	 *
	 * @returns void
	 */
	onLinkContextHiding: function() {
		delete this._link_index;
	},
	
	/**
	 * Returns object of the main Thunderbird's window
	 *
	 * @return {Components.interfaces.nsIXULWindow}
	 */
	getMessengerWin: function() {
		let noteObj = null;
		let winMan = CC["@mozilla.org/appshell/window-mediator;1"].getService();
		let winManInt = winMan.QueryInterface(CI.nsIWindowMediator);
		
		return winManInt.getMostRecentWindow("mail:3pane");
	}
	
}; // End of tbNoteDialog
