/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

// Namespace
if(typeof(extensions) == "undefined") extensions = {};
if(typeof(extensions.tbnotes) == "undefined") extensions.tbnotes = {};
if(typeof(extensions.tbnotes.notesView) == "undefined") extensions.tbnotes.notesView = {};

// Control object for NotesView
(function() {
  let CC = Components.classes;
  let CI = Components.interfaces;
  let CU = Components.utils;
  
  /**
   * Returns main Thunderbird's window - we needed it because of access 
   * to the tbNotes.
   * @private
   * @returns {Components.interfaces.nsIDOMWindow}
   */
  let getMsgWindow = function() {
    let winMan = CC["@mozilla.org/appshell/window-mediator;1"].getService();
    let winManInt = winMan.QueryInterface(CI.nsIWindowMediator);
    let win = winManInt.getMostRecentWindow("mail:3pane");
    
    return win;
  };
  
  /**
   * Edit note with given ID
   *
   * @param aNoteId {integer}
   * @returns {void}
   */
  this.editNote = function(aNoteId)
  {
    with(getMsgWindow()) {
      extensions.tbnotes.showEditNoteDialog(aNoteId, false);
    }
    // TODO Reload only if any note was really updated!
    window.location.reload();
  };
  
  /**
   * Delete note with given ID
   *
   * @param aNoteId {integer}
   * @returns {boolean}
   */
  this.deleteNote = function(aNoteId) {
    with(getMsgWindow()) {
      extensions.tbnotes.db.deleteNote(aNoteId);
    }
    // TODO Reload only if any note was really deleted!
    window.location.reload();
  };
  
  /**
   * ...
   *
   * @returns {void}
   */
  this.deleteSelecteNotes = function() {
    $(".singlenote").each(function(index, item) {
      let note_id = (new String(item.getAttribute("id"))).replace("note-", "");
      
      if(document.getElementById("id-check-" + note_id).checked) {
        with(getMsgWindow()) {
          extensions.tbnotes.db.deleteNote(note_id);
        }
        
        update = true;
      }
    });
    
    // TODO Update only when any note was really deleted!
    window.location.reload();
  };
  
  /**
   * Opens target of note with given ID
   *
   * @param aNoteId {integer}
   * @returns {boolean}
   */
  this.openTarget = function(aNoteId) {
    alert(aNoteId + ":" + aTargetType + ":" + aTargetKey);
  };
  
  /**
   * Returns object with array of notes that match current filter and count
   * of all notes.
   *
   * @returns {object} E.g. { filtered: [NotePrototype, ...], totalCount: 12 }
   */
  this.getNotes = function() {
    let notes = new Array();
    let total = 0;
    
    with(getMsgWindow()) {
      // TODO Get notes according to current filter!
      notes = extensions.tbnotes.db.getNotes();
      total = notes.length;
    }
    
    return { filtered: notes, totalCount: total };
  };
  
  this.showNoteText = function(aNoteId) {
    $("#note_" + aNoteId + "_showLink-span").hide();
    $("#note_" + aNoteId + "_hideLink-span").show();
    $("#note_" + aNoteId + "_body-div").show();
  };
  
  this.hideNoteText = function(aNoteId) {
    $("#note_" + aNoteId + "_showLink-span").show();
    $("#note_" + aNoteId + "_hideLink-span").hide();
    $("#note_" + aNoteId + "_body-div").hide();
  };
  
  /**
   * Returns preferences for the notes view.
   *
   * @returns {object}
   */
  this.getPrefs = function() {
    let viewPrefs = {}
    let onceAgain = false
    
    with(getMsgWindow()) {
      try {
        viewPrefs = {
          filter: {
            deleted: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.deleted"),
            updated: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.updated"),
            to_mail: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_mail"),
            to_address: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_address"),
            to_newsitem: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_newsitem"),
            to_feeditem: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_feeditem"),
            notarget: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.notarget")
          },
          show_texts: TBNOTES_PREFS.getBoolPref(TBNOTES_PREFS_KEY + "notesview.show_texts")
        };
      } catch(e) {
        // If there were any errors is commonly means that preferences are not saved yet so do it
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.deleted", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.updated", false);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_mail", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_address", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_newsitem", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.to_feeditem", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.filter.notarget", true);
        TBNOTES_PREFS.setBoolPref(TBNOTES_PREFS_KEY + "notesview.show_texts", false);
        
        onceAgain = true;
      }
    }
    
    return (onceAgain === true)
        ? extensions.tbnotes.notesView.getPrefs()
        : viewPrefs;
  };
  
  /**
   * Updates notes view
   *
   * @returns {void}
   */
  this.updateNotesView = function() {
    let prefs = extensions.tbnotes.notesView.getPrefs();
    let res = extensions.tbnotes.notesView.getNotes();
    let notes = res.filtered;
    
    // Update sidebar
    extensions.tbnotes.notesView.updateSidebar(notes.length, res.totalCount, prefs);
    
    // Update notes view
    extensions.tbnotes.notesView.updateNotesTable(notes, res.totalCount, prefs.show_texts);
  };
  
  /**
   * Updates notes view sidebar
   *
   * @param aDisplayed {integer}
   * @param aTotal {integer}
   * @param aOptions {object}
   * @returns {void}
   */
  this.updateSidebar = function(aDisplayed, aTotal, aOptions) {
    $("#displayed-notes-count").text(aDisplayed);
    $("#total-notes-count").text(aTotal);
    
    if(aOptions.filter.deleted == true) {
      $("#filter-deleted-check").attr("selected","true");
    }
    if(aOptions.filter.deleted == true) {
      $("#filter-deleted-check").attr("selected","true");
    }
    if(aOptions.filter.to_mail == true) {
      $("#filter-to_mail-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_address == true) {
      $("#filter-to_address-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_newsitem == true) {
      $("#filter-to_newsitem-check").attr("checked", "checked");
    }
    if(aOptions.filter.to_feeditem == true) {
      $("#filter-to_feeditem-check").attr("checked", "checked");
    }
    if(aOptions.filter.notarget == true) {
      $("#filter-notarget-check").attr("checked", "checked");
    }
    
    if(aOptions.show_texts == true) {
      $("#option-show_texts-check").attr("checked", "checked");
    }
  };
  
  /**
   * Updates notes table
   *
   * @param aNotes {array}
   * @param aTotalCount {integer}
   * @param aShowTexts {boolean}
   * @returns {void}
   */
  this.updateNotesTable = function(aNotes, aTotalCount, aShowTexts) {
    $("#notesview").html("");
    
    if(aNotes.length == 0) {
      let msg = (aTotalCount == 0)
          ? "There are no notes created yet. You have to create some before this page become usefull."
          : "Selected filter matches no notes. You can change the filter or <a href=\"#\">destroy it</a>.";
      
      $("#notesview-controls").hide();
      $("#notesview").append(
        '<tr class="singlenote">' +
        '  <th scope="row" class="check" rowspan="2"> </th>' +
        '  <td class="target"> </td>' + 
        '  <td class="updated"> </td>' + 
        '  <td class="title"><p><strong>' + msg + '</strong></p></td>' + 
        '</tr>' +
        '<tr><td colspan="3"> </td></tr>');
      
      return;
    }
    
    // Helper  method for formatting note's date
    let formatDate = function(aDate) {
      let p = aDate.split(" ");
      let d = p[0].split("-");
      //let t = p[1].split(":");
      return (new Number(d[2])) + "." + (new Number(d[1])) + ", " + d[0]/* + " " + 
             (new Number(t[0])) + ":" + (new Number(t[1]))*/;
    };
    
    for(let i=0; i<aNotes.length; i++) {
      let targetClass =
          (aNotes[i].mTargetType == "" || aNotes[i].mTargetType == null)
              ? "notarget"
              : aNotes[i].mTargetType;
      
      let targetAction =
          (targetClass != "notarget" && (aNotes[i].mTargetKey != "" || aNotes[i].mTargetKey != null))
              ? " onclick=\"extensions.tbnotes.notesView.openTarget('" + aNotes[i].mId + "', '" + aNotes[i].mTargetType + "', '" + aNotes[i].mTargetKey + "');\""
              : "";
      
      let targetName = (targetClass == "notarget") ? "no target" : aNotes[i].mTargetName;
      let target = "<span" + targetAction + " class=\"" + targetClass + "\"> </span>";
      
      $("#notesview").append(
        '<tr id="note-' + aNotes[i].mId + '" class="singlenote">' +
        '  <th scope="row" class="check" rowspan="2"><input type="checkbox" id="id-check-' + aNotes[i].mId + '" class="note-check"></input></th>' +
        '  <td class="target">' + target + '</td>' + 
        '  <td class="updated">' + formatDate(aNotes[i].mUpdated) + '</td>' +
        '  <td class="title">' +
        '    <p>' +
        '      <strong>' + aNotes[i].mTitle + '</strong> (' +
        '<span id="note_' + aNotes[i].mId + '_showLink-span" onclick="extensions.tbnotes.notesView.showNoteText(\'' + aNotes[i].mId + '\');" class="link">show</span>' +
        '<span id="note_' + aNotes[i].mId + '_hideLink-span" onclick="extensions.tbnotes.notesView.hideNoteText(\'' + aNotes[i].mId + '\');" class="link" style="display:none;">hide</span>' +
        ', <span id="note_' + aNotes[i].mId + '_editLink-span" onclick="extensions.tbnotes.notesView.editNote(\'' + aNotes[i].mId + '\');" class="link">edit</span>' +
        ', <span id="note_' + aNotes[i].mId + '_removeLink-span" onclick="extensions.tbnotes.notesView.deleteNote(\'' + aNotes[i].mId + '\');" class="link">remove</span>)' +
        '    </p>' +
        '  </td>' + 
        '</tr>' +
        '<tr>' +
        '  <td colspan="3">' +
        '    <div id="note_' + aNotes[i].mId + '_body-div" class="notebody" ' + ((aShowTexts == true) ? '' : ' style="display:none;"') + '>' +
        '      <pre>' + aNotes[i].mBody + '</pre>' +
        '    </div>' + 
        '  </td>' + 
        '</tr>');
    }
  };
  
  // =========================================================================
  // Initialize notes view
  
  $(function() {
    extensions.tbnotes.notesView.updateNotesView();
    
    // Check/uncheck all notes
    $("#checkallnotes-checkbox").click(function(aEvent) {
      if(this.checked) {
        $(".note-check").attr("checked", true);
      } else {
        $(".note-check").removeAttr("checked");
      }
    });
    
    // Show/hide full texts
    $("#option-show_texts-check").change(function(aEvent) {
      if(this.checked) { // Ensure that texts are shown
        $('div.notebody').show();
      } else { // Ensure that texts are hidden
        $('div.notebody').hide();
      }
    });
    
  });
  
}).apply(extensions.tbnotes.notesView)
