/* Copyright (c) 2009 Ondrej Donek.
   See the file LICENSE.txt for licensing information. */

/**
 * Main object that contains all main functionality of tbNotes.
 * 
 * @type Object
 */
(function() {
	

	
	// ======================================================================
  // Listeners
  
  /** 
   * Listener added to gMessageListeners (msgHdrViewOverlay.js)
   * 
   * @type Object
   */
  this.messageListener = {
    
    onStartHeaders: function() {
			var selMsg = extensions.tbnotes.getSelectedMessageDescription();
      var menuitem = document.getElementById("tbn-removenotesAction-menuitem");
      var hbox = document.getElementById("tbn-notesmsgheaderviewpane-hbox");
      
      if(selMsg == null) {
				return;
			}
			
			if(extensions.tbnotes.checkNoteForTargetKey(selMsg.key) === true) {
				// Selected message has any notes attached so display them.
				if(hbox) {
					extensions.tbnotes.removeChilds(hbox);
					
					var note = extensions.tbnotes.db.getNoteByTargetKey(selMsg.key);
					var label = document.createElementNS(XUL_NS, "label");
					label.setAttribute("value", ((note.mTitle == "") ? "No title" : note.mTitle));
					label.setAttribute("tooltiptext", "Edit '" + note.mTitle + "'.");
					label.setAttribute("class", "link");
					label.setAttribute("onclick", "extensions.tbnotes.showEditNoteDialog(" + note.mId + ");");
					hbox.appendChild(label);
				}
				
				// Update 'Remove all notes' command.
				if(menuitem) {
					if(menuitem.hasAttribute("disabled")) {
						menuitem.removeAttribute("disabled");
					}
				}
			} else {
				// No notes are attached to the selected message.
				if(hbox) {
					extensions.tbnotes.removeChilds(hbox);
					
					var label = document.createElementNS(XUL_NS, "label");
					label.setAttribute("value", "No notes");
					//label.setAttribute("style", "color: #666;");
					hbox.appendChild(label);
				}
				
				// Update 'Remove all notes' command.
				var menuitem = document.getElementById("tbn-removenotesAction-menuitem");
				if(menuitem) {
					menuitem.setAttribute("disabled", "true");
				}
			}
    },
    
    onBeforeShowHeaderPane: function() {},
    onEndHeaders: function() {},
    onEndAttachements: function() {}
  };
  
  /**
   * Listener for nsIMsgFolderNotificationService. 
   * 
   * @type Object
   */
  this.notificationListener = {
    
    /**
     * Notified after a command to delete a group of messages has been given, 
     * but before the messages have actually been deleted.
     * According to the current tbNotes preferences update notes attached 
     * to any of messages which going to be deleted.
     * 
     * @param aMsgs array
     * @return void
     */
    msgsDeleted: function(aMsgs) {
      CU.reportError("extensions.tbnotes.notificationListener.msgsDeleted:");
      CU.reportError(aMsgs);
    }
    
  };
  
	// ======================================================================
	// Event handlers
	
	/**
	 * Called when Thunderbird is loaded
	 * 
	 * @return void
	 */
	this.onLoad = function() 
  {
		// Initialize tbNotes SQLite storage
		extensions.tbnotes.db.init();
		
    // Add our notes panel to the Thunderbird's tabmail
    // Note: tbNotes currently has no tab implemented by self. For displaying 
    // view with all notes is used Thunderbird's inner browser. Implementation 
    // of notes view self is in chrome://tbnotes/content/notes-view.xhtml.
    //var tabmail = document.getElementById("tabmail");
    //if(tabmail) {
    //  tabmail.registerTabType(new tbNotesTabsPrototype());
    //}
    
		// Add new col to the main Thunderbird's messages tree
		var obSrv = CC["@mozilla.org/observer-service;1"].
				getService(CI.nsIObserverService);
		obSrv.addObserver(gTbNotesCreateDbObserver, "MsgCreateDBView", false);
		
    // Observe if any message is deleted
    //var notificationSrv = CC["@mozilla.org/messenger/msgnotificationservice;1"].
    //    getService(CI.nsIMsgFolderNotificationService);
    //notificationSrv.addListener(extensions.tbnotes.notificationListener); 
    
    // Add our listener to gMessagesListeners (defined in msgHdrViewOverlay.js)
    if(typeof(gMessageListeners) != "undefined") {
      gMessageListeners.push(extensions.tbnotes.messageListener);
    }
    
		// Add event listener for mail context menu
		var context = document.getElementById("mailContext");
		if(context) {
			context.addEventListener("popupshowing", extensions.tbnotes.onMailContextPopupShowing, false);
		}
    
	};
	
	/**
	 * Fired when Thunderbird going to be quit
	 * 
	 * @return void
	 */
	this.onUnload = function() 
  {
    try {
      var obSrv = CC["@mozilla.org/observer-service;1"].
          getService(CI.nsIObserverService);
      obSrv.removeObserver(gTbNotesCreateDbObserver, "MsgCreateDBView", false);
      
      //var notificationSrv = CC["@mozilla.org/messenger/msgnotificationservice;1"].
      //    getService(CI.nsIMsgFolderNotificationService);
      //notificationSrv.removeListener(extensions.tbnotes.notificationListener); 
      
      var context = document.getElementById("mailContext");
      if(context) {
        context.removeEventListener("popupshowing", extensions.tbnotes.onMailContextPopupShowing, false);
      }
    } catch(e) {
      alert(e);
    }
	};
  
	/**
	 * Fired when mail context popupmenu is showing.
	 * 
	 * @param aEvent object
	 * @return void
	 */
	this.onMailContextPopupShowing = function(aEvent) 
  {
		var enable_cmd = false;
		var menuitem1 = document.getElementById("tbn-editnote-menuitem");
		var menuitem2 = document.getElementById("tbn-removenote-menuitem");
		if(!menuitem1 || !menuitem2) {
			return;
		}
		
		// Check if any mail message is selected
		var selMsg = extensions.tbnotes.getSelectedMessageDescription();
		if(selMsg != null) {
			if(extensions.tbnotes.checkNoteForTargetKey(selMsg.key) === true) {
				enable_cmd = true;
			}
		}
		
		// Enable/disable "edit note" command
		if(enable_cmd === true) {
			if(menuitem1.hasAttribute("disabled")) {
				menuitem1.removeAttribute("disabled");
			}
			if(menuitem2.hasAttribute("disabled")) {
				menuitem2.removeAttribute("disabled");
			}
		} else {
			menuitem1.setAttribute("disabled", "true");
			menuitem2.setAttribute("disabled", "true");
		}
	};
	
	/**
	 * Performs command 
	 * 
	 * @param aCmdName {string}
   * @param aContext {string}
	 * @return void
	 */
	this.doCommand = function(aCmdName, aContext) 
  {
    var selMsg = extensions.tbnotes.getSelectedMessageDescription();
		var note = (selMsg != null) 
        ? extensions.tbnotes.db.getNoteByTargetKey(selMsg.key) 
        : null;

    
		switch(aCmdName) {
			// adding new note
			case "add_note":        
        if(note != null) {
          extensions.tbnotes.showEditNoteDialog(note);
        } else {
          extensions.tbnotes.showAddNoteDialog();
        }
				break;
			
			// edit selected note or edit note which is attached 
			// to the selected email message
			case "edit_note":
				if(note != null) {
					extensions.tbnotes.showEditNoteDialog(note);
				} else {
					// TODO Use stringbundles and prompts service!
          // TODO Mozna spise zobrazit confirm: "Nebyla nalezena zadna poznamka 
          //      k editaci"
					alert("There is no note to edit!");
				}
				break;
			
			// remove note from the selected message
			case "remove_note":
				if(note != null) {
					extensions.tbnotes.deleteNote(note);
				} else {
					// TODO Use stringbundles and prompt service!
					alert("There is no note to remove!");
				}
				break;
        
      // remove all notes attached to the selected message
			case "remove_notes":
        if(selMsg != null) {
          // TODO extensions.tbnotes.deleteNotesOfMessage(selMsg);
          alert("Sorry, but this command is not implemented yet!");
        }
        break;
      
      // show notes view
      case "show_notesview":
        var tabmail = document.getElementById("tabmail");
        if(tabmail) {
          tabmail.openTab("contentTab", { contentPage: TBNOTES_NOTESVIEW_URI });
        }
        break;
      
			default:
				CU.reportError("extensions.tbnotes.doCommand: Unknown command ['" + aCmdName + "']!");
				break;
		}
	};
	
	// ======================================================================
	// Normal methods
	
	/**
	 * Returns simple object describing currently selected mail message or NULL.
	 * 
	 * @return object|NULL
	 */
	this.getSelectedMessageDescription = function() 
  {
		if(gFolderDisplay.selectedMessage != null) {
		//if(gMessageDisplay.displayedMessage != null) {
			return {
				key: gFolderDisplay.selectedMessage.messageKey,
				subject: gFolderDisplay.selectedMessage.subject
			};
		}
		
		return null;
	};
	
	/**
	 * Returns TRUE if there is any note with given target key.
	 * 
	 * @param aKey {string}
	 * @return boolean
	 */
	this.checkNoteForTargetKey = function(aKey) 
  {
		return (extensions.tbnotes.db.getNoteByTargetKey(aKey) == null) ? false : true;
	};
	
	/**
	 * Show dialog for adding new note
	 * 
	 * @return void
	 */
	this.showAddNoteDialog = function() 
  {
		// Check if any message is currently selected
		var selMsg = extensions.tbnotes.getSelectedMessageDescription();
		
		// Open dialog self
		var params = { inn: { msg: selMsg }, out: null };
		window.openDialog(TBNOTES_NOTEDLG_URI, "", TBNOTES_NOTEDLG_FTRS, params).focus();
		
		if(params.out) {
			// params.out is now instance of `extensions.tbnotes.data.NotePrototype`.
			CU.reportError("Note: " + params.out.toString());
			params.out.save();
		}
	};
	
	/** 
	 * Shows dialog for editing note
	 * 
	 * @param aNote {object}
	 * @returned void
	 */
	this.showEditNoteDialog = function(aNote) 
  {
    if(typeof(aNote) != "object") {
      aNote = extensions.tbnotes.db.getNoteById(aNote);
    }
    
		var params = { inn: { note: aNote }, out: null };
		window.openDialog(TBNOTES_NOTEDLG_URI, "", TBNOTES_NOTEDLG_FTRS, params).focus();
		
		if(params.out) {
			// params.out is now instance of `extensions.tbnotes.data.NotePrototype`.
			CU.reportError("Note: " + params.out.toString());
			params.out.save();
			
			// TODO Check if this update is really needed at all time!
      extensions.tbnotes.messageListener.onStartHeaders();
		}
	};
	
	/**
	 * Mark note as deleted
	 * 
	 * @param aNote {Object}
	 * @returns {boolean}
	 */
	this.deleteNote = function(aNote) 
  {
    if(typeof(aNote) != "object") {
      aNote = extensions.tbnotes.db.getNoteById(aNote);
    }
		
		var pref = TBNOTES_PREFS_KEY + "dont_show_remove_confirm";
		
		// Check if user want to see confirmation dialog
		if(TBNOTES_PREFS.prefHasUserValue(pref) === true) {
			if(TBNOTES_PREFS.getBoolPref(pref) === true) {
				// Show confirmation if user really want to close unsaved note
				var prompts = CC["@mozilla.org/embedcomp/prompt-service;1"].
						getService(CI.nsIPromptService);
				
				// TODO Use stringbundles!
				var title = "Note dialog";
				var desc = "Do you really want to remove note '" + aNote.mTitle + "'?";
				var question = "Do not ask me again.";
				var check = { value: false };
				var res = prompts.confirmCheck(window, title, desc, question, check);
				
				TBNOTES_PREFS.setBoolPref(pref, check.value);
				
				if(res) { 
					aNote.remove();
				}
			}
		}
		
		return false;
  };
	
}).apply(extensions.tbnotes);
